@@ -1,11 +1,12 @@
 #include "common.h"
 #ifdef BUILD_X11
 #include <X11/Xlib.h>
-#include <X11/extensions/XShm.h>
 #include <X11/Xutil.h>
 #include <X11/extensions/shape.h>
+#include <X11/extensions/XShm.h>
 #include <sys/ipc.h>
 #include <sys/shm.h>
+
 #include "grab.h"
 #include "ximage.h"
 
@@ -15,8 +16,6 @@ static DATA8        rtab[256], gtab[256], btab[256];
 static void
 Tmp_HandleXError(Display * d, XErrorEvent * ev)
 {
-   d = NULL;
-   ev = NULL;
    _x_err = 1;
 }
 
@@ -95,6 +94,7 @@ __imlib_GrabXImageToRGBA(DATA32 * data, int ox, int oy, int ow, int oh,
                        tmp++;
                     }
                }
+             break;
           case 24:
           case 25:
           case 32:
@@ -528,7 +528,7 @@ __imlib_GrabDrawableToRGBA(DATA32 * data, int ox, int oy, int ow, int oh,
    char                is_pixmap = 0, created_mask = 0, is_shm = 0, is_mshm = 0;
    char                domask;
    int                 i;
-   int                 src_x, src_y, src_w, src_h, origw, origh;
+   int                 src_x, src_y, src_w, src_h;
    int                 width, height, clipx, clipy;
    XShmSegmentInfo     shminfo, mshminfo;
    XImage             *xim, *mxim;
@@ -537,8 +537,6 @@ __imlib_GrabDrawableToRGBA(DATA32 * data, int ox, int oy, int ow, int oh,
    domask = (pdomask) ? *pdomask : 0;
    /* FIXME: oh isn't used - i wonder if there's a bug looming... */
    oh = 0;
-   origw = w;
-   origh = h;
    if (grab)
       XGrabServer(d);
    XSync(d, False);
@@ -661,15 +659,9 @@ __imlib_GrabDrawableToRGBA(DATA32 * data, int ox, int oy, int ow, int oh,
      }
 
    /* Create an Ximage (shared or not) */
-   if (x_does_shm < 0)
-      __imlib_ShmCheck(d);
+   xim = __imlib_ShmGetXImage(d, v, p, xatt.depth, x, y, w, h, &shminfo);
+   is_shm = ! !xim;
 
-   xim = NULL;
-   if (x_does_shm)
-     {
-        xim = __imlib_ShmGetXImage(d, v, p, xatt.depth, x, y, w, h, &shminfo);
-        is_shm = ! !xim;
-     }
    if (!xim)
       xim = XGetImage(d, p, x, y, w, h, 0xffffffff, ZPixmap);
    if (!xim)
@@ -741,13 +733,16 @@ __imlib_GrabDrawableToRGBA(DATA32 * data, int ox, int oy, int ow, int oh,
 
    /* destroy the Ximage */
    if (is_shm)
-      __imlib_ShmDetach(d, &shminfo);
-   XDestroyImage(xim);
+      __imlib_ShmDestroyXImage(d, xim, &shminfo);
+   else
+      XDestroyImage(xim);
+
    if (mxim)
      {
         if (is_mshm)
-           __imlib_ShmDetach(d, &mshminfo);
-        XDestroyImage(mxim);
+           __imlib_ShmDestroyXImage(d, mxim, &mshminfo);
+        else
+           XDestroyImage(mxim);
      }
    if (created_mask)
       XFreePixmap(d, m);
